VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clError"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'*************************************************
'
' Error Class developed by Stuart Roberts 2001
' Email - stoorob@hotmail.com
' Website - www.software2try.com still under development but I'll get there.
'
' All comments about this or my other submitted code will be appreciated
'
'*************************************************

Private Declare Function pBGetFreeSystemResources Lib "rsrc32.dll" Alias "_MyGetFreeSystemResources32@4" (ByVal iResType As Integer) As Integer
Private Declare Function apiUserName Lib "advapi32.dll" Alias "GetUserNameA" (ByVal lpBuffer As String, nSize As Long) As Long
Private Declare Function apiCompName Lib "kernel32" Alias "GetComputerNameA" (ByVal lpBuffer As String, nSize As Long) As Long
Private Declare Function apiWindDir Lib "kernel32" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Private Declare Function apiSysDir Lib "kernel32" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Private Declare Function apiTempDir Lib "kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Private Declare Sub apiMemStatus Lib "kernel32" Alias "GlobalMemoryStatus" (lpBuffer As MEMORYSTATUS)

Private Type MEMORYSTATUS
        dwLength As Long
        dwMemoryLoad As Long
        dwTotalPhys As Long
        dwAvailPhys As Long
        dwTotalPageFile As Long
        dwAvailPageFile As Long
        dwTotalVirtual As Long
        dwAvailVirtual As Long
End Type

Private Const pvError = "Error.log" ' Name of file to write error descriptions to

Public Enum ErrorRequest
    Terminate = 1
    ResumeNext = 2
    Retry = 3
End Enum

'# Events to use with error class, need to declare your error variable in a form like below to enable these
'# Public WithEvents ErrorClass as new clError
Public Event NoErrorsExist(PropertyAccessed As String)
Public Event NoSuchError(Element As Integer, PropertyAccessed As String)

'# Property variables
Private m_Description() As String
Private m_Source() As String
Private m_Number() As Long
Private m_Additional() As String
Private m_ErrorCount As Long
Private m_StaticMessage As String
Private m_ErrorFile As String

'# Properties
Public Property Get Description() As String
    If m_ErrorCount > 0 Then
        Description = m_Description(m_ErrorCount - 1)
    Else
        Description = ""
        RaiseEvent NoErrorsExist("Description")
    End If
End Property

Private Property Let Description(aDescription As String)
    ReDim Preserve m_Description(m_ErrorCount - 1)
    m_Description(m_ErrorCount - 1) = aDescription
End Property

Public Property Get Source() As String
    If m_ErrorCount > 0 Then
        Source = m_Source(m_ErrorCount - 1)
    Else
        Source = ""
        RaiseEvent NoErrorsExist("Source")
    End If
End Property

Private Property Let Source(aSource As String)
    ReDim Preserve m_Source(m_ErrorCount - 1)
    m_Source(m_ErrorCount - 1) = aSource
End Property

Public Property Get Number() As Long
    If m_ErrorCount > 0 Then
        Number = m_Number(m_ErrorCount - 1)
    Else
        Number = 0
        RaiseEvent NoErrorsExist("Number")
    End If
End Property

Private Property Let Number(aNumber As Long)
    ReDim Preserve m_Number(m_ErrorCount - 1)
    m_Number(m_ErrorCount - 1) = aNumber
End Property

Public Property Get Additional() As String
    If m_ErrorCount > 0 Then
        Additional = m_Additional(m_ErrorCount - 1)
    Else
        Additional = ""
        RaiseEvent NoErrorsExist("Additional")
    End If
End Property

Private Property Let Additional(anAdditional As String)
    ReDim Preserve m_Additional(m_ErrorCount - 1)
    m_Additional(m_ErrorCount - 1) = anAdditional
End Property

Public Property Get ErrorCount() As Long
    ErrorCount = m_ErrorCount
End Property

Public Property Get StaticMessage() As String
    StaticMessage = m_StaticMessage
End Property

Public Property Let StaticMessage(aMessage As String)
    m_StaticMessage = aMessage
End Property

Public Property Get ErrorFile() As String
    ErrorFile = m_ErrorFile
End Property

Public Property Let ErrorFile(aFile As String)
    If Trim(aFile) > "" Then
        m_ErrorFile = Trim(aFile)
    Else
        m_ErrorFile = pvError
    End If
End Property

Public Function GetDescription(Occurrence As Integer) As String
    If Occurrence > m_ErrorCount Then
        GetDescription = ""
        RaiseEvent NoSuchError(Occurrence, "Description")
    Else
        GetDescription = m_Description(Occurrence - 1)
    End If
End Function

Public Function GetSource(Occurrence As Integer) As String
    If Occurrence > m_ErrorCount Then
        GetSource = ""
        RaiseEvent NoSuchError(Occurrence, "Source")
    Else
        GetSource = m_Source(Occurrence - 1)
    End If
End Function

Public Function GetNumber(Occurrence As Integer) As Long
    If Occurrence > m_ErrorCount Then
        GetNumber = ""
        RaiseEvent NoSuchError(Occurrence, "Number")
    Else
        GetNumber = m_Number(Occurrence - 1)
    End If
End Function

Public Sub ClearAllErrors()
    
    ReDim m_Description(0)
    ReDim m_Source(0)
    ReDim m_Number(0)
    m_ErrorCount = 0
    
End Sub

'*************************************************************
'Procedure:    Public Method LogError
'Created on:   02/16/01
'Module:       clError
'Module File:  clError.cls
'Project:      WMConvert
'Project File: WMExport.vbp
'Parameters:
'NewDescription
'NewSource
'NewNumber
'Optional DisplayError
'Optional AdditionalMessage
'Optional RealError
'Description:  Logs the error supplied to a text file and displays to screen if DisplayError is set to TRUE.
'              RealError is to determine whether to show user Abort/Retry/Ignore or just OK button in message box.
'*************************************************************
Public Function LogError(NewDescription As String, NewSource As String, NewNumber As Long, _
                         Optional DisplayError As Boolean = False, Optional AdditionalMessage As String = "", _
                         Optional RealError As Boolean = True) As ErrorRequest
'# No error handling in this or any methods called within or the error variable Err will reset to no error which
'# the calling function probably doesn't want
    Dim nErrorFile As Integer, nTimes As Integer
    Dim bUsed As Byte
    Dim lTotal As Long, lFree As Long
    Dim sResources As String, sTimes As String
    Dim MsgOption As Integer
    
    If RealError Then
        MsgOption = vbAbortRetryIgnore
    Else
        MsgOption = vbOKOnly
    End If
    LogError = ResumeNext
    m_ErrorCount = m_ErrorCount + 1
    Description = NewDescription
    Source = NewSource
    Number = NewNumber
    Additional = AdditionalMessage
    
    FreeMemory bUsed, lTotal, lFree
    sResources = CStr(pBGetFreeSystemResources(0)) & "%"
    
    If DisplayError Then
        nTimes = ErrorNumberOccurred(NewNumber)
        If nTimes > 0 Then
            sTimes = vbCrLf & vbCrLf & "This error has occurred " & nTimes
            If nTimes = 1 Then
                sTimes = sTimes & " time"
            Else
                sTimes = sTimes & " times"
            End If
            sTimes = sTimes & " so far during this session."
        End If
        If StaticMessage > "" Then
            sTimes = vbCrLf & vbCrLf & StaticMessage & sTimes
        End If
        If Additional > "" Then
            sTimes = vbCrLf & vbCrLf & "Additional information relating to the error :" & vbCrLf & Additional & sTimes
        End If
        Select Case MsgBox("An error has occurred with the following message : " & NewDescription & vbCrLf & _
                           "from within : " & NewSource & vbCrLf & _
                           "with an error number of : " & CStr(NewNumber) & vbCrLf & vbCrLf & _
                           "Total Memory   : " & lTotal & vbCrLf & _
                           "Free Memory    : " & lFree & vbCrLf & _
                           "Free Resources : " & sResources & sTimes, vbInformation Or MsgOption, App.Title)
            Case vbAbort
                LogError = Terminate
            Case vbRetry
                LogError = Retry
            Case vbIgnore
                LogError = ResumeNext
            Case Else
                LogError = ResumeNext
        End Select
    End If
' Write latest error to end of error file
    nErrorFile = FreeFile
    Open App.Path & "\" & ErrorFile For Append Lock Read Write As #nErrorFile
    
    Print #nErrorFile, String(30, "*")
    Print #nErrorFile, ""
    Print #nErrorFile, "User Name : " & UserName
    Print #nErrorFile, "Computer Name : " & ComputerName
    Print #nErrorFile, "Total Memory : " & CStr(lTotal)
    Print #nErrorFile, "Total Free Memory : " & CStr(lFree)
    Print #nErrorFile, "System Resources Free : " & sResources
    Print #nErrorFile, "Temporary Directory is : " & TempDir
    Print #nErrorFile, "Windows Directory is : " & WinDir
    Print #nErrorFile, "System Directory is : " & SystemDir
    Print #nErrorFile, ""
    Print #nErrorFile, "Error Number has occurred : " & NewNumber
    Print #nErrorFile, "with an error description of : " & NewDescription
    Print #nErrorFile, "and a source of : " & NewSource
    Print #nErrorFile, ""
    Print #nErrorFile, "Time of error was " & CStr(Time)
    Print #nErrorFile, "and the Date was  " & CStr(Date)
    If Additional > "" Then
        Print #nErrorFile, ""
        Print #nErrorFile, "Additional information relating to error :"
        Print #nErrorFile, Additional
    End If
    If StaticMessage > "" Then
        Print #nErrorFile, ""
        Print #nErrorFile, StaticMessage
    End If
    Print #nErrorFile, ""
    Print #nErrorFile, "Times occurred during current session : " & nTimes
    Print #nErrorFile, ""
    Print #nErrorFile, String(30, "*")
    Print #nErrorFile, ""
    
    Close #nErrorFile
    
End Function

Public Function ErrorNumberOccurred(ErrorNumber As Long) As Integer
    Dim nLoop As Integer
    
    ErrorNumberOccurred = 0
    If m_ErrorCount > 0 Then
        For nLoop = 0 To m_ErrorCount - 1
            If m_Number(nLoop) = ErrorNumber Then
                ErrorNumberOccurred = ErrorNumberOccurred + 1
            End If
        Next
    End If
End Function

Public Function ErrorSourceOccurred(ErrorSource As String) As Integer
    Dim nLoop As Integer
    
    ErrorSourceOccurred = 0
    If m_ErrorCount > 0 Then
        For nLoop = 0 To m_ErrorCount - 1
            If UCase(Trim(m_Source(nLoop))) = UCase(Trim(ErrorSource)) Then
                ErrorSourceOccurred = ErrorSourceOccurred + 1
            End If
        Next
    End If
End Function

Public Function ErrorDescriptionOccurred(ErrorDescription As String) As Integer
    Dim nLoop As Integer
    
    ErrorDescriptionOccurred = 0
    If m_ErrorCount > 0 Then
        For nLoop = 0 To m_ErrorCount - 1
            If UCase(Trim(m_Description(nLoop))) = UCase(Trim(ErrorDescription)) Then
                ErrorDescriptionOccurred = ErrorDescriptionOccurred + 1
            End If
        Next
    End If
End Function

Private Function StripTerminator(ByVal strString As String) As String
    Dim intZeroPos As Integer
    
    intZeroPos = InStr(strString, Chr$(0))
    If intZeroPos > 0 Then
        StripTerminator = Left$(strString, intZeroPos - 1)
    Else
        StripTerminator = strString
    End If
    
End Function

Private Function UserName() As String
    Dim Bufstr As String
    
    Bufstr = Space$(50)
    If apiUserName(Bufstr, 50) > 0 Then
        UserName = Bufstr
        UserName = RTrim(UserName)
        UserName = StripTerminator(UserName)
    Else
        UserName = ""
    End If
    
End Function

Public Function ComputerName() As String
    Dim Bufstr As String
    
    Bufstr = Space$(50)
    If apiCompName(Bufstr, 50) > 0 Then
        ComputerName = Bufstr
        ComputerName = RTrim(ComputerName)
        ComputerName = StripTerminator(ComputerName)
    Else
        ComputerName = ""
    End If
    
End Function

Public Function TempDir() As String
    Dim Bufstr As String
    
    Bufstr = Space$(50)
    If apiTempDir(50, Bufstr) > 0 Then
        TempDir = Bufstr
        TempDir = RTrim(TempDir)
        TempDir = StripTerminator(TempDir)
        
        If Right$(TempDir, 1) <> "\" Then
            TempDir = TempDir + "\"
        End If
        
    Else
        TempDir = ""
    End If
    
End Function

Public Function WinDir() As String
    Dim Bufstr As String
    Bufstr = Space$(50)
    
    If apiWindDir(Bufstr, 50) > 0 Then
        WinDir = Bufstr
        WinDir = RTrim(WinDir)
        WinDir = StripTerminator(WinDir)
        
        If Right$(WinDir, 1) <> "\" Then
            WinDir = WinDir + "\"
        End If
        
    Else
        WinDir = ""
    End If
    
End Function

Public Function SystemDir() As String
    Dim Bufstr As String
    Bufstr = Space$(50)
    
    If apiSysDir(Bufstr, 50) > 0 Then
        SystemDir = Bufstr
        SystemDir = RTrim(SystemDir)
        SystemDir = StripTerminator(SystemDir)
        
        If Right$(SystemDir, 1) <> "\" Then
            SystemDir = SystemDir + "\"
        End If
        
    Else
        SystemDir = ""
    End If
    
End Function

Public Sub FreeMemory(ByRef btePercentUsed As Byte, ByRef lngTotalRam As Long, ByRef lngFreeRam As Long)
    Dim Memory As MEMORYSTATUS
    
    Memory.dwLength = 32
    
    apiMemStatus Memory
    btePercentUsed = Memory.dwMemoryLoad
    lngTotalRam = Memory.dwTotalPhys / 1024
    lngFreeRam = Memory.dwAvailPhys / 1024
End Sub

Private Sub Class_Initialize()
On Error Resume Next
    m_ErrorCount = 0
    ErrorFile = pvError
End Sub

Private Sub Class_Terminate()
On Error Resume Next
    ReDim m_Description(0)
    ReDim m_Source(0)
    ReDim m_Number(0)
    ReDim m_Additional(0)
End Sub
